"use strict";

import { createHighlighterCore } from '@shikijs/core';
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript';
import React, { useEffect, useState } from 'react';
import { ScrollView, StyleSheet, Text, View } from 'react-native';
import { trimNewlines } from 'trim-newlines';
import { getRNStylesFromShikiStyle } from './../../utils/style';
import { jsx as _jsx } from "react/jsx-runtime";
const SyntaxHighlighter = ({
  code,
  language,
  languageMap,
  theme,
  textStyle,
  scrollViewProps,
  containerStyle
}) => {
  const [highlightedNodes, setHighlightedNodes] = useState(null);
  const [stylesheet, setStylesheet] = useState(null);
  useEffect(() => {
    const highlightCode = async () => {
      if (!languageMap[language]) {
        console.warn(`Warning: Language "${language}" not found in the language map. Falling back to "plaintext".`);
      }

      // Ensure that the theme is loaded and processed only once or when necessary
      if (!stylesheet) {
        const processedStyle = await loadThemeAndGetStyles(theme); // Utility function to convert theme to RN styles
        setStylesheet(processedStyle);
      }
      const highlighter = await createHighlighterCore({
        engine: createJavaScriptRegexEngine(),
        langs: Object.values(languageMap)
      });
      const tokens = await highlighter.codeToHast(code, {
        lang: language,
        theme
      });
      setHighlightedNodes(tokens.children || null);
    };
    highlightCode();
  }, [code, language, languageMap, theme, stylesheet]);
  const getStylesForNode = node => {
    if (!stylesheet) return [];
    const classes = node.properties?.className ?? [];
    return classes.map(c => stylesheet[c]).filter(Boolean);
  };
  const renderNode = (nodes, keyPrefix = 'row') => nodes.reduce((acc, node, index) => {
    const keyPrefixWithIndex = `${keyPrefix}_${index}`;
    if (node.children) {
      const styles = StyleSheet.flatten([textStyle, {
        color: stylesheet?.shiki?.color
      }, ...getStylesForNode(node)]);
      acc.push(/*#__PURE__*/_jsx(Text, {
        style: styles,
        children: renderNode(node.children, `${keyPrefixWithIndex}_child`)
      }, keyPrefixWithIndex));
    }
    if (node.value) {
      acc.push(trimNewlines(String(node.value)));
    }
    return acc;
  }, []);
  return /*#__PURE__*/_jsx(ScrollView, {
    ...scrollViewProps,
    horizontal: true,
    contentContainerStyle: [stylesheet?.shiki, scrollViewProps?.contentContainerStyle, containerStyle],
    children: /*#__PURE__*/_jsx(View, {
      onStartShouldSetResponder: () => true,
      children: highlightedNodes && renderNode(highlightedNodes)
    })
  });
};
export default SyntaxHighlighter;

// Example utility function to load and convert theme
async function loadThemeAndGetStyles(themeName) {
  const theme = await import(`shiki/dist/themes/${themeName}.mjs`);
  const rnStyle = getRNStylesFromShikiStyle(theme.default);
  return rnStyle;
}
//# sourceMappingURL=SyntaxHighlighter.js.map