import type { HighlighterCore, LanguageInput } from '@shikijs/core'
import type { FunctionComponent, ReactNode } from 'react'
import type { ScrollViewProps, TextStyle, ViewStyle } from 'react-native'
import type { HighlighterStyleSheet } from './../../utils/style'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { useEffect, useState } from 'react'

import { ScrollView, StyleSheet, Text, View } from 'react-native'
import { trimNewlines } from 'trim-newlines'
import { getRNStylesFromShikiStyle } from './../../utils/style'

interface RendererNode {
  type: string
  value?: string
  children?: RendererNode[]
  properties?: {
    className?: string[]
  }
}

export interface SyntaxHighlighterProps {
  code: string
  language: string // Valid language input
  languageMap: Record<string, LanguageInput>
  theme: string // Theme name or settings
  textStyle?: TextStyle
  scrollViewProps?: ScrollViewProps
  containerStyle?: ViewStyle
  children?: ReactNode
}

const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ code, language, languageMap, theme, textStyle, scrollViewProps, containerStyle }) => {
  const [highlightedNodes, setHighlightedNodes] = useState<RendererNode[] | null>(null)
  const [stylesheet, setStylesheet] = useState<HighlighterStyleSheet | null>(null)

  useEffect(() => {
    const highlightCode = async () => {
      if (!languageMap[language]) {
        console.warn(`Warning: Language "${language}" not found in the language map. Falling back to "plaintext".`)
      }

      // Ensure that the theme is loaded and processed only once or when necessary
      if (!stylesheet) {
        const processedStyle = await loadThemeAndGetStyles(theme) // Utility function to convert theme to RN styles
        setStylesheet(processedStyle)
      }

      const highlighter: HighlighterCore = await createHighlighterCore({
        engine: createJavaScriptRegexEngine(),
        langs: Object.values(languageMap),
      })

      const tokens = await highlighter.codeToHast(code, {
        lang: language,
        theme,
      })

      setHighlightedNodes(tokens.children || null)
    }

    highlightCode()
  }, [code, language, languageMap, theme, stylesheet])

  const getStylesForNode = (node: RendererNode): TextStyle[] => {
    if (!stylesheet)
      return []

    const classes: string[] = node.properties?.className ?? []
    return classes.map((c: string) => stylesheet[c]).filter(Boolean) as TextStyle[]
  }

  const renderNode = (nodes: RendererNode[], keyPrefix = 'row'): ReactNode[] =>
    nodes.reduce<ReactNode[]>((acc, node, index) => {
      const keyPrefixWithIndex = `${keyPrefix}_${index}`
      if (node.children) {
        const styles = StyleSheet.flatten([textStyle, { color: stylesheet?.shiki?.color }, ...getStylesForNode(node)])
        acc.push(
          <Text style={styles} key={keyPrefixWithIndex}>
            {renderNode(node.children, `${keyPrefixWithIndex}_child`)}
          </Text>,
        )
      }

      if (node.value) {
        acc.push(trimNewlines(String(node.value)))
      }

      return acc
    }, [])

  return (
    <ScrollView {...scrollViewProps} horizontal contentContainerStyle={[stylesheet?.shiki, scrollViewProps?.contentContainerStyle, containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{highlightedNodes && renderNode(highlightedNodes)}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter

// Example utility function to load and convert theme
async function loadThemeAndGetStyles(themeName: string): Promise<HighlighterStyleSheet> {
  const theme = await import(`shiki/dist/themes/${themeName}.mjs`)
  const rnStyle = getRNStylesFromShikiStyle(theme.default)
  return rnStyle
}
